<?php
/**
 * CFixedAssetManager class file.
 *
 * A version of the asset manager that do not copy the file used as assets
 */


class CFixedAssetManager extends CAssetManager
{
	/**
	 * @var array published assets
	 */
	private $_published=array();
	/**
	 * @var string base web accessible path for storing private files
	 */
	private $_basePath;
	
	public function setBasePath($value)
	{
		if(($basePath=realpath($value))!==false && is_dir($basePath) )
			$this->_basePath=$basePath;
		else
			throw new CException(Yii::t('yii','CAssetManager.basePath "{path}" is invalid. Please make sure the directory exists.',
				array('{path}'=>$value)));
	}
	public function getBasePath()
	{
		return $this->_basePath;
	}
	
	/**
	 * Publishes a file or a directory.
	 * This method will NOT copy the specified asset to a web accessible directory
	 * it must exist already
	 */
	public function publish($path,$hashByName=false,$level=-1,$forceCopy=false)
	{
		if(isset($this->_published[$path]))
			return $this->_published[$path];
		else if(($src=realpath($path))!==false)
		{
			if(is_file($src))
			{
				$dir=$this->hash($hashByName ? basename($src) : dirname($src));
				$fileName=basename($src);
				$dstDir=$this->getBasePath().DIRECTORY_SEPARATOR.$dir;
				$dstFile=$dstDir.DIRECTORY_SEPARATOR.$fileName;
				
				if ( file_exists($dstFile) )
					return $this->_published[$path]=$this->getBaseUrl()."/$dir/$fileName";
					
				$dst = $dstFile;
			}
			else if(is_dir($src))
			{
				$dir=$this->hash($hashByName ? basename($src) : $src);
				$dstDir=$this->getBasePath().DIRECTORY_SEPARATOR.$dir;

				if( is_dir($dstDir) )
					return $this->_published[$path]=$this->getBaseUrl().'/'.$dir;
				$dst = $dstDir;
			}
		}
		throw new CException(Yii::t('app','The asset \'{asset}\' to be published to \'{dir}\' does not exist.',
			array('{asset}'=>$path,  '{dir}'=>$dst)));
	}

		/**
	 * Returns the URL of a published file path.
	 * This method does not perform any publishing. It merely tells you
	 * if the file path is published, what the URL will be to access it.
	 * @param string directory or file path being published
	 * @param boolean whether the published directory should be named as the hashed basename.
	 * If false, the name will be the hashed dirname of the path being published.
	 * Defaults to false. Set true if the path being published is shared among
	 * different extensions.
	 * @return string the published URL for the file or directory. False if the file or directory does not exist.
	 */
	public function getPublishedUrl($path,$hashByName=false)
	{
		if(isset($this->_published[$path]))
			return $this->_published[$path];
		if(($path=realpath($path))!==false)
		{
			if(is_file($path))
				return $this->getBaseUrl().'/'.$this->hash($hashByName ? basename($path) : dirname($path)).'/'.basename($path);
			else
				return $this->getBaseUrl().'/'.$this->hash($hashByName ? basename($path) : $path);
		}
		else
			return false;
	}
	
	/**
	 * Generate a CRC32 hash for the directory path. Collisions are higher
	 * than MD5 but generates a much smaller hash string.
	 * the hash do not depends on the yii version string (using '1.1.2' as to generate the same hash
	 * as before the changes
	 * @param string string to be hashed.
	 * @return string hashed string.
	 */
	protected function hash($path)
	{
		return sprintf('%x',crc32($path."1.1.2"));
	}
}
