<?php

class FeedsController extends CController
{
	/**
	 * @var string specifies the default action to be 'list'.
	 */
	public $defaultAction='list';
    
    public function filters()
    {
        return array(
            'accessControl',
        );
    }
    public function accessRules()
    {
        return array(
            array('allow',
                'actions'=>array('create', 'modify', 'delete', 'rename', 'copy' ),
                'users'=> Yii::app()->user->addUserByRoles('feeds'),
            ),
            array('allow',
                'actions'=>array('edit', 'list' ),
                'users'=> Yii::app()->user->addUserByRoles('edit'),
            ),
            array('deny',
                'actions'=>array('create', 'modify', 'delete', 'rename', 'copy', 'list', 'edit' ),
            ),

        );
    }
    
    private function select() {

        $feeds = Feeds::findAll($crit);
        if ( Yii::app()->request->isAjaxRequest )
            $this->renderPartial('select', array(  'source' => $feeds ) );
        else
            $this->render('select', array(  'source' => $feeds ) );
    }
    
	public function actionList()
	{
        $feeds = Feeds::findAll();
        if ( Yii::app()->request->isAjaxRequest )
            $this->renderPartial('list', array( 'feeds' => $feeds ));
        else {
            $this->render('list', array( 'feeds' => $feeds ));
            Yii::app()->user->returnUrl = array('/feeds/list');
        }
        
	}
    
    public function actionEdit()
	{
        if ( isset($_POST['action']) && $_POST['action']=='Cancel' ) {
            $this->redirect( Yii::app()->user->returnUrl );
            return;
        }
        if ( isset($_POST['rename']) ) {
            $file = new Feeds( $_GET ); 
            $file->name = $_POST['rename'];
            $file->rename();
        } else if ( $_POST['action']=='SaveAs' ) {
            $file = new Feeds( $_GET ); 
            $file->name = $_POST['Feeds']['name'];
            $file->copy();
        }
        if ( isset( $_POST['data'] ) ) {
            if ( !isset($file) )
                $file = new Feeds( $_GET );
            $file->setData( $_POST['data'] );
            if ( $file->validate() ) {
				if ( $file->save() )
					$this->redirect(  Yii::app()->user->returnUrl );
			}
            
        }elseif ( isset($_GET['id']) ) {
            $file = new Feeds( $_GET );

            $crit = new CDbCriteria;
            $crit->condition = "type=:type";
            
            $crit->params = array( ":type"=>"project" );
            $projects = FileItem::findAll($crit);
            $crit->params = array( ":type"=>"playlist" );
            $playlists = FileItem::findAll($crit);
            $crit->params = array( ":type"=>"image" );
            $images = FileItem::findAll($crit);
            $crit->params = array( ":type"=>"video" );
            $videos = FileItem::findAll($crit);
            $crit->params = array( ":type"=>"audio" );
            $sounds = FileItem::findAll($crit);
            
            $this->render('edit', array('feeds' => $file, 
                                        'project' => $projects, 
                                        'playlist' => $playlists,
                                        'image' => $images,
                                        'video' => $videos,
                                        'audio' => $sounds,
                                        'title' => Yii::t('app', "Edit datafeed"),
                                        'savebutton'=>true
                                        ) );
        } else 
            $this->select();
        
	}
    
    public function actionCreate()
	{
        $feeds = new Feeds;
		
		if (isset($_POST['Feeds'])) {
			$feeds->setScenario('create');
			$feeds->setAttributes( $_POST['Feeds'] );

			if ( $feeds->validate() ) {
				if ( $feeds->save() )
					$this->redirect(array('modify', "id" => $feeds->id));
			}
		}
		$this->render('create',array('feeds'=>$feeds));
        
	}
    
    public function actionModify()
	{
        if ( isset($_POST['action']) && $_POST['action']=='Cancel' ) {
            $this->redirect( Yii::app()->user->returnUrl );
            return;
        }
        
        if ( isset($_POST['id']) ) $_GET['id']=$_POST['id'];
   		if ( isset($_POST['FeedsEntry']) ) {
            $file = new Feeds( $_GET );
            $file->entry = $_POST['FeedsEntry'];
            if ( $file->validate() ) {
				if ( $file->save() )
					$this->redirect( Yii::app()->user->returnUrl );
                else
                    $this->render('modify',array('feeds'=>$file));
			} else
                $this->render('modify',array('feeds'=>$file));
        } elseif ( isset($_GET['id']) ) {
            $file = new Feeds( $_GET );
            
            $this->render('modify',array('feeds'=>$file, 'title'=>Yii::t('app', 'Modify ').$file->name));
            
        } else 
            $this->select();
        
	}
    
    public function actionDelete()
	{
        if ( isset($_POST['id']) ) $_GET['id']=$_POST['id'];
   		if ( isset($_GET['id']) ) {
            $file = new Feeds( $_GET );
            if ( !$file->delete() )
                throw new CHttpException(500, Yii::t('app', 'The requested feed does not exist or cannot be deleted.'));
            if ( Yii::app()->request->isAjaxRequest ) $this->actionList();
            else $this->redirect(array('list'));
        } else 
            $this->select();
        
	}
    
    protected function copyRename( $action ) {
        if ( isset($_POST['id']) ) $_GET['id']=$_POST['id'];
        if ( isset( $_POST['Feeds'] ) ) {
            $file = new Feeds( $_GET );
            $file->scenario = 'rename';
            $file->attributes = $_POST['Feeds'];
            if ( $file->validate() ) {
                if ( $action=="Rename" ) {
                    if ( !$file->rename() )
                        throw new CHttpException(500,Yii::t('app', 'The requested file does not exist or cannot be renamed.'));
                } else {
                    if ( !$file->copy() )
                        throw new CHttpException(500,Yii::t('app', 'The requested file does not exist or cannot be copied.'));
                }
                $this->redirect(array('list'));
            } else
                $this->render('rename', array( 'file' => $file,) );
        } elseif ( isset($_GET['id']) ) {
            $file = new Feeds( $_GET );
            $this->render('rename', array( 'name' => "$action {$file->name}", 'action'=> $action, 'file' => $file,) );
        } else {
            $file = new Feeds( );
            $feeds = Feeds::findAll();
            $this->render('rename', array( 'name' => $action, 'action'=> $action, 'file' => $file, 'feeds' => $feeds ) );
        }
    
    }
    public function actionRename()
	{
        $this->copyRename( Yii::t('app', "Rename") );
	}
    
    public function actionCopy()
	{
        $this->copyRename( Yii::t('app', "Copy") );
	}
    
}