<?php
class SerialConfig extends CFormModel {
	
    var $confpathsys="/usr/share/raperca/comportfsm/";
    var $confpathusr="/srv/raperca/comportfsm/";
    var $powersavefile="power_save.fsm";

    var $baudRateSet= array( 110, 300, 600, 1200, 2400, 4800, 9600, 14400, 19200, 38400, 57600, 115200, 128000, 256000 );
    var $paritySet=array("none", "even", "odd");
    var $flowControlSet=array("none", "XonXoff", "RTS/CTS", "RS485");
    var $byteSizeSet=array("7", "8");
    var $stopBitsSet=array("0", "1");

    var $monPower=false;
    var $onTime="00:00";
    var $offTime="24:00";
    var $powerSave="true";
    var $href="";
    var $portNumber=1;

    var $comSettings="false";
    var $baudRate=9600;
    var $byteSize=8;
    var $stopBits=1;
    var $parity="none";
    var $flowControl="none";
    
    var $uploadedfile;
    
    var $modified = null;
 
    var $playerConf = null;
    public function init() {
        if ( Yii::app()->user->info->model=='Bonsai' ) 
            $this->portNumber=1;
        else
            $this->portNumber=2;
    }
    public function rules() {
		$res = array(
            array( "enablePowerSave, fixedMonPower", 'safe', 'on'=>'io, display, backup' ),
            array( 'onTime, offTime', 'checkTime', 'on'=>'io, display, backup' ),
            array( 'href', 'safe', 'on'=>'io, backup' ),
            array( 'enableComSettings, baudRate, byteSize, stopBits, parity', 'safe', 'on'=>'io, backup' ),
            array( 'uploadedfile', 'file', 'types'=>'fsm', 'on'=>'upload' ),
            
    	);
        return $res;
	}
    public function attributeLabels()
	{
		return array(
            'enablePowerSave'=>'Enable display power saving',
            'fixedMonPower'=>'Use fixed schedule for display power',
            'onTime'=>'Turn monitor ON',
            'offTime'=>'Turn monitor OFF',
            'href'=>'Protocols',
            'enableComSettings'=>'Modify COM port settings',
            'baudRate'=>'Baud rate',
            'byteSize'=>'Data bits',
            'parity'=>'Parity',
            'stopBits'=>'Stop bits',
            
		);
	}
    
    public function checkTime( $attribute, $params  ) {
        if ( !preg_match('/(([01][0-9]|2[0-3]):[0-5][0-9]|24:00)/', $this->$attribute) ) {
            $this->addError( $attribute, "Time format is HH:MM (between 00:00 and 24:00)");
        }
    }
    
    function toList($in) {
        $name = $in.'Set';
        $ret = array();
        foreach ($this->$name as $val)
            $ret[$val]=$val;
        return $ret;
    }
    
    function getBackupType(){
        if ( $this->href == "[disabled]" || $this->href == "" )
            return "disabled";
        if ( strpos( $this->href, $this->confpathsys )!==false )
            return "system";
        else 
            return "inline";
    }

    function getFileContent() {
        if ( strpos( $this->href, $this->confpathusr )!==false ) {
            return file_get_contents( $this->href ) ;
        }
        return "";
    }
    
    function getBackupName(){
        if ( $this->href == "[disabled]" || $this->href == "" )
            return "[disabled]";
        if ( strpos( $this->href, $this->confpathsys )!==false ) {
            return substr( $this->href, strlen($this->confpathsys) );
        } else {
            return substr( $this->href, strlen($this->confpathusr) );
        }
    }
    
    function setBackup($type, $name="", $content=null ){
		if ( $type=='disabled' ) {
			$this->href = "[disabled]";
			$this->modified = true;
			return;
		}
		if ( $name=="" ){
			$this->addError('href', 'A name must be provided for the protocol.');
			return;
		}
        if ( $type=='system' ) {
            $this->href = $this->confpathsys . $name;
        }else {
            $this->href = $this->confpathusr . $name;
            if ( $content=="" ){
                if ( !file_exists( $this->href ) )
                    $this->addError('', 'Protocol do not exist. Must provide the protocol file.');
            } else {           
                file_put_contents($this->href, $content);
            }
        }
        $this->modified = true;
    }
    function setEnablePowerSave( $val ) {
        if ( $val ) $this->powerSave="true";
        else $this->powerSave="false";
    }
    function getEnablePowerSave( ) {
        return ( $this->powerSave=="true" );
    }
    
    function setFixedMonPower( $val ) {
        if ( $val ) $this->monPower="true";
        else $this->monPower="false";
    }
    function getFixedMonPower( ) {
        return ( $this->monPower=="true" );
    }
    
    function setEnableComSettings( $val ) {
        if ( $val ) $this->comSettings="true";
        else $this->comSettings="false";
    }
    function getEnableComSettings( ) {
        return ( $this->comSettings=="true" );
    }
    
    function setAttributes( $inputs, $safeOnly=true ) {

        
        if ( $this->modified===null) 
            $this->load();
        if ( isset( $inputs['href'] ) && $inputs['href']=="" )
            $inputs['href'] = "[disabled]";
            
		$values = $this->getSafeAttributeNames();
        
        foreach ($values as $str )
			if ( isset( $inputs[$str] ) && (!isset($this->$str) || $inputs[$str]!=$this->$str) ){
				if ( isset($this->$str) )
					$this->modified=true;
				$this->$str = $inputs[$str];
			}

		return $this->modified;
	}
	
	function load( ) {
        if ( $this->playerConf===null ) $this->playerConf = new PlayerConfig;
        if ( !$this->playerConf->loadConfig() ) return false;
        
        $domtree = $this->playerConf->domdoc;
        
		$ComPorts=$domtree->getElementsByTagName("serialPort");
		if ($ComPorts->length>0){
			$ComPorts=$ComPorts->item(0);
			
			if ($ComPorts->hasAttribute("enablePowerSave"))
				$this->powerSave=$ComPorts->getAttribute("enablePowerSave"); 
			if ($ComPorts->hasAttribute("href")) 
				$this->href=$ComPorts->getAttribute("href");
			
			if ( $this->href=="" ) {
				$this->href = "[disabled]";
				$this->powerSave="false";
			}elseif ($this->href==$this->confpathsys.$this->powersavefile){
				$this->href="[disabled]";
				$this->powerSave="true";
			}
			
			if ($ComPorts->hasAttribute("onTime") && $ComPorts->hasAttribute("offTime")) {
				$this->onTime=$ComPorts->getAttribute("onTime");
				$this->offTime=$ComPorts->getAttribute("offTime");
			}
			if ( $this->onTime!="00:00" || $this->offTime!="24:00" ){
				$this->monPower="true";
			}
			
			
			if ($ComPorts->hasAttribute("portNumber")) $this->portNumber=$ComPorts->getAttribute("portNumber");
			
			if ($ComPorts->hasAttribute("baudRate")) { $this->baudRate=$ComPorts->getAttribute("baudRate"); $this->comSettings="true";}
			if ($ComPorts->hasAttribute("byteSize")) { $this->byteSize=$ComPorts->getAttribute("byteSize"); $this->comSettings="true";}
			if ($ComPorts->hasAttribute("stopBits")) { $this->stopBits=$ComPorts->getAttribute("stopBits"); $this->comSettings="true";}
			if ($ComPorts->hasAttribute("flowControl")) { $this->flowControl=$ComPorts->getAttribute("flowControl"); $this->comSettings="true";}
			if ($ComPorts->hasAttribute("parity")) { $this->parity=$ComPorts->getAttribute("parity"); $this->comSettings="true";}
			
			
		} else {
			$this->href = "[disabled]";
			$this->powerSave="false";
		}
        $this->modified = false;
	}
	
	function save( ) {
        
        if ( !$this->modified ) return false;
        
        if ( $this->playerConf===null ) $this->playerConf = new PlayerConfig;
        if ( !$this->playerConf->loadConfig() ) return false;
        $domtree = $this->playerConf->domdoc;
        
        $ComPorts=$domtree->getElementsByTagName("serialPort");
		
		if ( $this->href=="[disabled]")
			$this->href="";
		if (  $this->powerSave=="true" && $this->href==""){
			$this->href = $this->confpathsys.$this->powersavefile;
		}
		if ( $this->href!="" ) {
			if ($ComPorts->length>0){
				$ComPorts=$ComPorts->item(0);
			} else {
				$ComPorts=$domtree->createElement("serialPort");	
				$player=$domtree->documentElement->getElementsByTagName("player");
				if ($player->length>0) {
					$ComPorts=$player->item(0)->appendChild($ComPorts);
				}else{
					$ComPorts=$domtree->documentElement->appendChild($ComPorts);
					$this->addError( "href", "Warning no player tag found in the config, cannot add SerialPort tag.<br/>");
				}
			}
			
			$ComPorts->setAttribute( "href", $this->href );
            if ( $this->monPower!==false ) {
                if ( $this->monPower=="true" && ($this->onTime!="00:00" || $this->offTime!="24:00")){
                    $ComPorts->setAttribute( "onTime", $this->onTime );
                    $ComPorts->setAttribute( "offTime", $this->offTime );
                } else {
                    $this->monPower="false";
                    $ComPorts->removeAttribute("onTime");
                    $ComPorts->removeAttribute("offTime");
                }
            }
			$ComPorts->setAttribute( "enablePowerSave", $this->powerSave );
			
			$ComPorts->setAttribute("portNumber",$this->portNumber);
			if ( $this->comSettings=="true" ){
				$ComPorts->setAttribute("baudRate",$this->baudRate);
				$ComPorts->setAttribute("byteSize",$this->byteSize);
				$ComPorts->setAttribute("stopBits",$this->stopBits);
				$ComPorts->setAttribute("flowControl",$this->flowControl);
				$ComPorts->setAttribute("parity",$this->parity);
			} else {
				$ComPorts->removeAttribute("baudRate");
				$ComPorts->removeAttribute("byteSize");
				$ComPorts->removeAttribute("stopBits");
				$ComPorts->removeAttribute("flowControl");
				$ComPorts->removeAttribute("parity");
			}
		} else {
			if ($ComPorts->length>=0){
				$deleteList=array();
				foreach($ComPorts as $item) $deleteList[]=$item;
				foreach($deleteList as $item) $item->parentNode->removeChild($item);
			}
		}
        $this->playerConf->saveConfig( true );
        if ( $this->getScenario()!='io' )
            Yii::app()->user->tools->addReason( "power save config change" );
        else
            Yii::app()->user->tools->addReason( "serial port config change" );
		return true;
	}
	
	function getFileList() {
		$filelist1=scandir($this->confpathsys);
		$filelist=array("[disabled]" => "[disabled]");
		foreach ($filelist1 as $file){
			if ($file=="." || $file==".." || $file==$this->powersavefile) continue;
			$filelist[$this->confpathsys.$file]="[sys] ".basename($file,".fsm");
		}
	    $filelist2=scandir($this->confpathusr);
	    foreach ($filelist2 as $file){
			if ($file!="." && $file!="..") $filelist[$this->confpathusr.$file]=basename($file,".fsm");
	    }
		return $filelist;
	}
	
	function saveFsm( $file ) {
        if ( !$file->saveAs( $this->confpathusr. $file->name ) )
            $this->addError('uploadedfile', 'Saving the fsm file failed');
	}
    
}